@@ -92,6 +92,21 @@ sub union {
   return $rel->create($coldb,undef,%opts);
 }
 
+##==============================================================================
+## Relation API: dbinfo
+
+## \%info = $rel->dbinfo($coldb)
+##  + embedded info-hash for $coldb->dbinfo()
+sub dbinfo {
+  my ($rel,$coldb) = @_;
+  $rel = $rel->fromDB($coldb);
+  my $info = $rel->SUPER::dbinfo();
+  my @keys = qw(ddcServer ddcTimeout ddcLimit ddcSample dmax cfmin);
+  @$info{@keys} = @$rel{@keys};
+  return $info;
+}
+
+
 ##==============================================================================
 ## Relation API: profiling & comparison: top-level
 
@@ -134,12 +149,23 @@ sub profile {
   }
   undef $result12; ##-- save some memory
 
+  ##-- check whether count-query uses any token-attributes
+  my $needKeys = grep {UNIVERSAL::can($_,'getMatchId') && $_->getMatchId==2} @{$qcount->getKeys->getExprs};
+
   ##-- get raw g2 results and update slice-wise profiles
   my $fcoef  = $opts{fcoef};
-  my $qkeys2 = DDC::XS::CQKeys->new($qcount);
-  $qkeys2->setOptions($qcount->getDtr->getOptions);
-  $qkeys2->SetMatchId(2);
-  my $qcount2 = DDC::XS::CQCount->new($qkeys2, $qcount->getKeys, $qcount->getSample, $qcount->getSort, $qcount->getLo, $qcount->getHi);
+  my ($qcount2);
+  if ($needKeys) {
+    my $qkeys2 = DDC::XS::CQKeys->new($qcount);
+    $qkeys2->setOptions($qcount->getDtr->getOptions);
+    $qkeys2->SetMatchId(2);
+    $qcount2 = DDC::XS::CQCount->new($qkeys2, $qcount->getKeys, $qcount->getSample, $qcount->getSort, $qcount->getLo, $qcount->getHi);
+  }
+  else {
+    my $qdtr2 = DDC::XS::CQTokAny->new;
+    $qdtr2->setOptions($qcount->getDtr->getOptions);
+    ($qcount2 = $qcount->clone())->setDtr($qdtr2);
+  }
   my $result2 = $rel->ddcQuery($coldb, $qcount2, limit=>-1, logas=>'f2');
   foreach (@{$result2->{counts_}}) {
     next if (!defined($prf=$y2prf{$y=$_->[1]}));
@@ -149,18 +175,26 @@ sub profile {
   undef $result2; ##-- save some memory
 
   ##-- query independent f1 and update slice-wise profiles
-  my $qcount1 = $qcount2->clone();
-  $_->setMatchId(1) foreach (grep {UNIVERSAL::isa($_,'DDC::XS::CQCountKeyExprToken') && $_->getMatchId==2}
-			     @{$qcount1->getDtr->getQCount->getKeys->getExprs},
-			     @{$qcount1->getKeys->getExprs},
-			    );
-  $qcount1->getDtr->setMatchId(1);
-  my $result1 = $rel->ddcQuery($coldb, $qcount1, limit=>-1, logas=>'f1');
-  foreach (@{$result1->{counts_}}) {
-    next if (!defined($prf=$y2prf{$y=$_->[1]}));
-    $prf->{f1} += $_->[0]*$fcoef;
+  if ($needKeys) {
+    my $qcount1 = $qcount2->clone();
+    $_->setMatchId(1) foreach (grep {UNIVERSAL::isa($_,'DDC::XS::CQCountKeyExprToken') && $_->getMatchId==2}
+			       @{$qcount1->getDtr->getQCount->getKeys->getExprs},
+			       @{$qcount1->getKeys->getExprs},
+			      );
+    $qcount1->getDtr->setMatchId(1);
+    my $result1 = $rel->ddcQuery($coldb, $qcount1, limit=>-1, logas=>'f1');
+    foreach (@{$result1->{counts_}}) {
+      next if (!defined($prf=$y2prf{$y=$_->[1]}));
+      $prf->{f1} += $_->[0]*$fcoef;
+    }
+    undef $result1; ##-- save some memory (but not much)
+  } else {
+    foreach $prf (values %y2prf) {
+      my $f1=0;
+      $f1 += $_ foreach (values %{$prf->{f12}});
+      $prf->{f1} = $f1*$fcoef;
+    }
   }
-  undef $result1; ##-- save some memory (but not much)
 
   ##-- query independent N
   my $qstrN   = "count(* #sep)";
@@ -211,7 +245,7 @@ sub profile {
 					   titles   => \@titles,
 					   qinfo    => $qinfo,
 					  );
-  $mp->trim(%opts);
+  $mp->trim(%opts, empty=>!$opts{fill});
 
   ##-- return
   return $mp;
@@ -279,7 +313,7 @@ sub ddcQuery {
   my $logas = $opts{logas} // 'ddcQuery';
   my $level = exists($opts{loglevel}) ? $opts{loglevel} : $coldb->{logProfile};
 
-  my $qstr = ref($query) ? $query->toString : $query;
+  my $qstr = ref($query) ? $query->toStringFull : $query;
   my $cli  = $rel->ddcClient();
   $cli->{limit} = $opts{limit}//-1 if (exists($opts{limit}));
 
@@ -363,7 +397,7 @@ sub countQuery {
     push(@qnodes2,$_) if ($_->getMatchId==2);
   }
   $rel->logconfess("no primary target-nodes found in daughter query '", $qdtr->toString, "': use match-id =1 to specify primary target(s)")
-    if (!@qnodes1);
+    if (!@qnodes1 && !@qnodes2);
   $_->setMatchId(1) foreach (@qnodes1);
 
   ##-- check for target match-id =2 and maybe implicitly construct near() query
@@ -457,14 +491,16 @@ sub countQuery {
     }
     ++$xi;
   }
-  $qtconds->setMatchId(2);
+  #$qtconds //= DDC::XS::CQTokAny->new();
+  $qtconds->setMatchId(2) if ($qtconds);
   my $qtemplate = $qdtr->clone->mapTraverse(sub {
 					      my $nod = shift;
 					      if (UNIVERSAL::can($nod,'getMatchId') && $nod->getMatchId==2) {
-						return $qtconds;
+						return $qtconds // $nod;
 					      }
 					      return $nod;
 					    });
+  $qtemplate->setOptions($qdtr->getOptions->clone) if (!$qtemplate->getOptions); ##-- traversal bug
   $qtemplate->getOptions->setFilters([@$filters,@qtfilters]);
 
   ##-- cleanup coldb parser (so we're using "real" refcounts)
